/*  Copyright (c) 2009 by Alex Leone <acleone ~AT~ gmail.com>

    This file is part of the Arduino TLC5940 Library.

    The Arduino TLC5940 Library is free software: you can redistribute it
    and/or modify it under the terms of the GNU General Public License as
    published by the Free Software Foundation, either version 3 of the
    License, or (at your option) any later version.

    The Arduino TLC5940 Library is distributed in the hope that it will be
    useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with The Arduino TLC5940 Library.  If not, see
    <http://www.gnu.org/licenses/>. */

#ifndef SPARKFUN_TLC5940_H
#define SPARKFUN_TLC5940_H

#include <stdint.h>

// ----------------------------------------------------------------------------

#define NUM_TLCS    2
#define TLC_PWM_PERIOD    8192
#define TLC_GSCLK_PERIOD    3

/** SS will be set to output as to not interfere with SPI master operation.
    If you have changed the pin-outs and the library doesn't seem to work
    or works intermittently, make sure this pin is set correctly.  This pin
    will not be used by the library other than setting its direction to
    output. */
#define TLC_SS_PIN       PB2
#define TLC_SS_DDR       DDRB

/** OC2B (Arduino digital pin 3) -> GSCLK (TLC pin 18) */
#define GSCLK_PIN    PD3
#define GSCLK_PORT   PORTD
#define GSCLK_DDR    DDRD

/** OC1A (Arduino digital pin 9) -> XLAT (TLC pin 24) */
#define XLAT_PIN     PB1
#define XLAT_PORT    PORTB
#define XLAT_DDR     DDRB

/** OC1B (Arduino digital pin 10) -> BLANK (TLC pin 23) */
#define BLANK_PIN    PB2
#define BLANK_PORT   PORTB
#define BLANK_DDR    DDRB

/** SIN (TLC pin 26) */
#define SIN_PIN        PB3
#define SIN_PORT       PORTB
#define SIN_DDR        DDRB

/** SCLK (TLC pin 25) */
#define SCLK_PIN       PB5
#define SCLK_PORT      PORTB
#define SCLK_DDR       DDRB

// ----------------------------------------------------------------------------

/** Enables the Timer1 Overflow interrupt, which will fire after an XLAT
    pulse */
#define set_XLAT_interrupt()    TIFR1 |= _BV(TOV1); TIMSK1 = _BV(TOIE1)

/** Disables any Timer1 interrupts */
#define clear_XLAT_interrupt()  TIMSK1 = 0

/** Enables the output of XLAT pulses */
#define enable_XLAT_pulses()    TCCR1A = _BV(COM1A1) | _BV(COM1B1)

/** Disables the output of XLAT pulses */
#define disable_XLAT_pulses()   TCCR1A = _BV(COM1B1)

extern volatile uint8_t tlc_needXLAT;
extern uint8_t tlc_GSData[NUM_TLCS * 24];

class Tlc5940
{
  public:
    void init(uint16_t initialValue = 0);
    void clear(void);
    uint8_t update(void);
    void set(uint8_t channel, uint16_t value);
    uint16_t get(uint8_t channel);
    void setAll(uint16_t value);
};

void tlc_shift8_init(void);
void tlc_shift8(uint8_t byte);

extern Tlc5940 Tlc;

#endif /* SPARKFUN_TLC5940_H */

