/*  Copyright (c) 2024 by soruce <soruce ~ AT ~ soruce.net>

    This file is part of the Arduino PimientoClockLib Library.

    The Arduino PimientoClockLib Library is free software: you can redistribute it
    and/or modify it under the terms of the GNU General Public License as
    published by the Free Software Foundation, either version 3 of the
    License, or (at your option) any later version.

    The Arduino PimientoClockLib Library is distributed in the hope that it will be
    useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with The Arduino PimientoClockLib Library.  If not, see
    <http://www.gnu.org/licenses/>.
*/

/**
 * This program displays a simple scrolling test pattern.
 * Pressing the left or right button will decrease or increase the scrolling speed.
*/

#include <PimientoClockLib.h>
#include <Streaming.h>

bool stateInitNeeded = true;
uint32_t stateUpdateCounter = 0;

extern void updateState(uint8_t event = EventSystemUpdate);

// ----

void setup() {
  pimiento.init(updateState);
}

void loop() {
  pimiento.loop();
}

// ----

uint16_t scrollSpeed = 250;

void updateState(uint8_t event) {
    if (event == EventSystemUpdate) PimientoClockLib::everyMillis(scrollSpeed, stateUpdateCounter, loopDrawTestPattern2);
    else if (event == EventLeftButtonReleased) incSpeed(-50);
    else if (event == EventRightButtonReleased) incSpeed(50);
}

void incSpeed(int8_t increment) {
  scrollSpeed += increment;
  scrollSpeed = constrain(scrollSpeed, 50, 1000);
}

void loopDrawTestPattern2() {

  if (stateInitNeeded) {
    display.clearAll();
    display.drawHorizontalLine(0, 0, 16, 127);
    display.drawHorizontalLine(1, 1, 14, 64);
    display.drawHorizontalLine(2, 2, 12, 32);
    display.drawHorizontalLine(3, 3, 10, 16);
    display.drawHorizontalLine(4, 4, 8, 8);
    display.drawHorizontalLine(5, 5, 6, 4);
    display.drawHorizontalLine(6, 6, 4, 2);
    display.drawHorizontalLine(7, 7, 2, 1);

    display.drawHorizontalLine(16 + 0, 0, 16, 127);
    display.drawHorizontalLine(16 + 1, 1, 14, 64);
    display.drawHorizontalLine(16 + 2, 2, 12, 32);
    display.drawHorizontalLine(16 + 3, 3, 10, 16);
    display.drawHorizontalLine(16 + 4, 4, 8, 8);
    display.drawHorizontalLine(16 + 5, 5, 6, 4);
    display.drawHorizontalLine(16 + 6, 6, 4, 2);
    display.drawHorizontalLine(16 + 7, 7, 2, 1);
    stateInitNeeded = false;
  }

  display.moveFrameBufferContentsDown();
  display.moveFrameBufferContentsRight();
}
